Package: VanillaICE
Version: 1.74.0
Title: A Hidden Markov Model for high throughput genotyping arrays
Authors@R: person("Robert", "Scharpf", email="rscharpf@jhu.edu", role=c("aut", "cre"))
Description: Hidden Markov Models for characterizing chromosomal alteration in high throughput SNP arrays.
Date: 2021-11-21
Depends: R (>= 3.5.0), BiocGenerics (>= 0.13.6), GenomicRanges (>=
        1.27.6), SummarizedExperiment (>= 1.5.3)
Imports: MatrixGenerics, Biobase, S4Vectors (>= 0.23.18), IRanges (>=
        1.14.0), oligoClasses (>= 1.31.1), foreach, matrixStats,
        data.table, grid, lattice, methods, GenomeInfoDb (>= 1.11.4),
        crlmm, tools, stats, utils, BSgenome.Hsapiens.UCSC.hg18
Suggests: RUnit, human610quadv1bCrlmm
Collate: 'AllClasses.R' 'AllGenerics.R' 'datasets.R' 'functions.R'
        'help.R' 'hmm-methods.R' 'methods-ArrayViews.R'
        'methods-CopyNumScanParams.R' 'methods-EmissionParam.R'
        'methods-FilterParam.R' 'methods-HMM.R' 'methods-HMMList.R'
        'methods-HmmGRanges.R' 'methods-HmmParam.R'
        'methods-HmmTrellisParam.R' 'methods-IdiogramParams.R'
        'methods-LogLik.R' 'methods-SnpArrayExperiment.R'
        'methods-SnpDataFrame.R' 'methods-TransitionParam.R'
        'methods-Viterbi.R' 'updates.R' 'zzz.R'
Enhances: doMC, doMPI, doSNOW, doParallel, doRedis
License: LGPL-2
LazyLoad: yes
biocViews: CopyNumberVariation
Roxygen:
RoxygenNote: 7.1.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/VanillaICE
git_branch: RELEASE_3_23
git_last_commit: 7a1caf4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 23:50:20 UTC; biocbuild
Author: Robert Scharpf [aut, cre]
Maintainer: Robert Scharpf <rscharpf@jhu.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 05:04:50 UTC; unix
