# Generated by roxygen2: do not edit by hand

export("baf_means<-")
export("baf_sds<-")
export("cn_means<-")
export("cn_sds<-")
export("emission<-")
export("emissionParam<-")
export("lrrFile<-")
export()
export(ArrayViews)
export(CopyNumScanParams)
export(EMupdates)
export(EmissionParam)
export(FilterParam)
export(HMM)
export(HMMList)
export(HmmParam)
export(HmmTrellisParam)
export(IdiogramParams)
export(LogLik)
export(NA_filter)
export(SnpArrayExperiment)
export(SnpExperiment)
export(SnpGRanges)
export(TransitionParam)
export(acf2)
export(bafFile)
export(baf_means)
export(baf_sds)
export(baumWelchUpdate)
export(calculateEmission)
export(cn_means)
export(cn_sds)
export(cnvFilter)
export(cnvSegs)
export(colModes)
export(deletion)
export(doUpdate)
export(dropDuplicatedMapLocs)
export(dropSexChrom)
export(duplication)
export(emission)
export(emissionParam)
export(filters)
export(getExampleSnpExperiment)
export(getHmmParams)
export(gtFile)
export(hemizygous)
export(hmm2)
export(homozygous)
export(isHeterozygous)
export(lrrFile)
export(numberFeatures)
export(parseSourceFile)
export(parsedPath)
export(probability)
export(rescale)
export(rowMAD)
export(rowModes)
export(segs)
export(snpArrayAssays)
export(sourcePaths)
export(sweepMode)
export(threshold)
export(updateHmmParams)
export(viewports)
export(xygrid)
export(xyplotList)
exportClasses(ArrayViews)
exportClasses(CopyNumScanParams)
exportClasses(FilterParam)
exportClasses(HMM)
exportClasses(HMMList)
exportClasses(IdiogramParams)
exportClasses(LogLik)
exportClasses(SnpArrayExperiment)
exportClasses(SnpGRanges)
exportClasses(matrixOrNULL)
exportMethods("$")
exportMethods("$<-")
exportMethods("colnames<-")
exportMethods(baf)
exportMethods(colnames)
exportMethods(copyNumber)
exportMethods(dim)
exportMethods(genotypes)
exportMethods(length)
exportMethods(lrr)
exportMethods(ncol)
exportMethods(nrow)
exportMethods(plot)
exportMethods(sapply)
exportMethods(show)
exportMethods(start)
exportMethods(state)
import(BiocGenerics)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(SummarizedExperiment)
import(foreach)
import(grid)
import(lattice)
import(methods)
import(oligoClasses)
importFrom(Biobase,assayDataElement)
importFrom(Biobase,assayDataElementNames)
importFrom(Biobase,assayDataNew)
importFrom(Biobase,rowMax)
importFrom(crlmm,calculateRBaf)
importFrom(data.table,fread)
importFrom(matrixStats,colMedians)
importFrom(matrixStats,rowMedians)
importFrom(oligoClasses,baf)
importFrom(oligoClasses,lrr)
importFrom(stats,acf)
importFrom(stats,dnorm)
importFrom(stats,dunif)
importFrom(stats,pnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(tools,file_path_sans_ext)
importFrom(utils,packageDescription)
importFrom(utils,read.table)
importMethodsFrom(Biobase,assayData)
importMethodsFrom(Biobase,fData)
importMethodsFrom(Biobase,featureData)
importMethodsFrom(Biobase,featureNames)
importMethodsFrom(Biobase,pData)
importMethodsFrom(Biobase,phenoData)
importMethodsFrom(Biobase,sampleNames)
importMethodsFrom(Biobase,snpCallProbability)
importMethodsFrom(oligoClasses,baf)
importMethodsFrom(oligoClasses,copyNumber)
importMethodsFrom(oligoClasses,lrr)
importMethodsFrom(oligoClasses,state)
useDynLib(VanillaICE)
